﻿//===============================================================================
// Microsoft patterns & practices
// Guidance on Building a Release Pipeline with Team Foundation Server
//===============================================================================
// Copyright © Microsoft Corporation.  All rights reserved.
// This code released under the terms of the 
// Microsoft patterns & practices license (http://dataguidance.codeplex.com/license)
//===============================================================================


namespace TreyResearch.WpfClient
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Windows;
    using System.Windows.Controls;
    using TreyResearch.WpfClient.SensorReadingServiceRef;

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : IDisposable
    {
        private readonly SensorReadingServiceClient _service;
        private readonly Random _readingGenerator;
        private const int MaxNum = 10000;
        private int _currentPage = 1;
        private const double PageSize = 5;
        private readonly Guid _clientId = Guid.NewGuid();

        public MainWindow()
        {
            InitializeComponent();
            _readingGenerator = new Random();
            _service = new SensorReadingServiceClient();
        }

        private void LoadSensorReadings()
        {
            ToggleNetworkStatus(isActive: true);
            UpdateControls(_service.GetSensorReadingsPage(_clientId.ToString(), _currentPage));
            ToggleNetworkStatus(isActive: false);
        }

        private void ToggleNetworkStatus(bool isActive)
        {
            NetworkActivityPost.IsIndeterminate = isActive;
            SensorReadingTabsContainer.IsEnabled = !isActive;
        }

        private void OnSubmitButtonClick(object sender, RoutedEventArgs e)
        {
            ToggleNetworkStatus(isActive: true);
            PostData();
            RandomReading.Text = string.Format(CultureInfo.CurrentCulture, "{0}", _readingGenerator.Next(0, MaxNum));
            ToggleNetworkStatus(isActive: false);
        }

        private void PostData()
        {
            _service.StoreSensorReading(new SensorReading()
                {
                    Id = Convert.ToInt32(RandomReading.Text),
                    CreateDate = DateTime.Now,
                    ClientId = _clientId.ToString()
                });
        }

        private void UpdateControls(dynamic resultPage)
        {
            var totalCount = Convert.ToInt32(resultPage.TotalCount);
            var totalNumberOfPages = (totalCount <= PageSize)
                ? 1
                : Math.Ceiling(totalCount / PageSize);

            PageInfo.Text = string.Format(CultureInfo.CurrentCulture,
                Properties.Resources.PagerText,
                resultPage.TotalCount,
                _currentPage,
                totalNumberOfPages);

            Prev.IsEnabled = _currentPage > 1;
            Next.IsEnabled = _currentPage < totalNumberOfPages;

            SensorReadingList.ItemsSource = resultPage.Results;
        }

        private readonly Dictionary<string, int> _buttonPageActions = new Dictionary<string, int>
            {
                {"Prev",-1},
                {"Next",1},
            };

        private void OnBrowsePageClick(object sender, RoutedEventArgs e)
        {
            var button = (Button)sender;

            var key = button.Name;
            var increment = _buttonPageActions.ContainsKey(key) ? _buttonPageActions[key] : 0;
            _currentPage += increment;

            LoadSensorReadings();
        }

        private void SensorReadingTabsContainer_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (GetReadingsPage.IsSelected)
            {
                LoadSensorReadings();
            }
            else
            {
                RandomReading.Text = string.Format(CultureInfo.CurrentCulture, "{0}", _readingGenerator.Next(0, MaxNum));
            }
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!disposing) return;
            
            // dispose managed resources
            _service.Close();
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
